from datetime import datetime, timedelta

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import RegexURLResolver, Resolver404
from django.utils.encoding import smart_str

from secretballot.models import Vote


# Base Item Class
class Item(object):
    def __init__(self, request, title, default, group=None, *args, **kwargs):
        self.request = request
        self.title = title
        self.default = default
        self.group = group


class GetItem(Item):
    def __init__(self, request, title, get, field_name='', base_url=None, \
            default=False, *args, **kwargs):
        get['value'] = str(get['value'])
        self.get = get
        self.field_name = field_name
        self.base_url = base_url
        super(GetItem, self).__init__(request, title, default, *args, **kwargs)

    def is_active(self, request):
        if hasattr(self, 'get'):
            if self.get['name'] in request.GET:
                return request.GET[self.get['name']] == self.get['value']

        return False

    def modify(self, view):
        """
        adds the get item as extra context
        """
        view.params['extra_context'][self.get['name']] = self.get['value']
        return view

    def get_absolute_url(self):
        addition_pairs = [(self.get['name'], self.get['value']), ]
        remove_keys = ['page', ]
        q = dict([(k, v) for k, v in self.request.GET.items()])
        for key in remove_keys:
            if key in q:
                del q[key]
        for key, value in addition_pairs:
            if key:
                if value:
                    q[key] = value
                else:
                    q.pop(key, None)
            qs = '&'.join(['%s=%s' % (k, v) for k, v in q.items()])

        get_string = '?' + qs if len(q) else ''
        if self.base_url:
            return "%s%s" % (self.base_url, get_string)
        else:
            return get_string


class URLPatternItem(Item):
    def __init__(self, request, title, path, matching_pattern_names, default):
        self.path = path
        self.matching_pattern_names = matching_pattern_names
        super(URLPatternItem, self).__init__(request, title, default)

    def resolve_pattern_name(self, resolver, path):
        tried = []
        match = resolver.regex.search(path)
        if match:
            new_path = path[match.end():]
            for pattern in resolver.url_patterns:
                try:
                    sub_match = pattern.resolve(new_path)
                except Resolver404, e:
                    sub_tried = e.args[0].get('tried')
                    if sub_tried is not None:
                        tried.extend([(pattern.regex.pattern + \
                                '   ' + t) for t in sub_tried])
                    else:
                        tried.append(pattern.regex.pattern)
                else:
                    if sub_match:
                        sub_match_dict = dict([(smart_str(k), v) for \
                                k, v in match.groupdict().items()])
                        sub_match_dict.update(resolver.default_kwargs)
                        for k, v in sub_match[2].iteritems():
                            sub_match_dict[smart_str(k)] = v
                        try:
                            return pattern.name
                        except AttributeError:
                            return self.resolve_pattern_name(pattern, new_path)
                    tried.append(pattern.regex.pattern)
            raise Resolver404({'tried': tried, 'path': new_path})
        raise Resolver404({'path': path})

    def is_active(self, request):
        urlconf = getattr(request, "urlconf", settings.ROOT_URLCONF)
        resolver = RegexURLResolver(r'^/', urlconf)

        url_name = self.resolve_pattern_name(resolver, request.path)
        return url_name in self.matching_pattern_names

    def modify(self, view):
        return view

    def get_absolute_url(self):
        return self.path


# Specific Items
class CalEntryUpcomingItem(GetItem):
    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].upcoming()
        return view


class CalEntryNext7DaysItem(GetItem):
    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].next7days()
        return view


class CalEntryThisWeekendItem(GetItem):
    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].thisweekend()
        return view


class CalEntryThisMonthItem(GetItem):
    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].thismonth()
        return view


class IntegerFieldRangeItem(GetItem):
    def __init__(self, request, title, get, field_name, filter_range, \
            default=False):
        self.filter_range = filter_range
        super(IntegerFieldRangeItem, self).__init__(
            request=request,
            title=title,
            get=get,
            field_name=field_name,
            default=default
        )

    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].filter(
            **{"%s__range" % self.field_name: self.filter_range}
        )
        return view


class MostRecentItem(GetItem):
    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].order_by(
            '-%s' % self.field_name
        )
        return view


class MostLikedItem(GetItem):
    def modify(self, view):
        queryset = view.params['queryset']
        view.params['queryset'] = queryset.extra(
            select={
                'vote_score': '(SELECT COUNT(*) from %s WHERE vote=1 AND \
object_id=%s.%s AND content_type_id=%s) - (SELECT COUNT(*) from %s WHERE \
vote=-1 AND object_id=%s.%s AND content_type_id=%s)' % (
                    Vote._meta.db_table,
                    queryset.model._meta.db_table,
                    queryset.model._meta.pk.attname,
                    ContentType.objects.get_for_model(queryset.model).id,
                    Vote._meta.db_table,
                    queryset.model._meta.db_table,
                    queryset.model._meta.pk.attname,
                    ContentType.objects.get_for_model(queryset.model).id
                )
            }
        ).order_by('-vote_score')
        return view


class PagingCountItem(GetItem):
    def modify(self, view):
        try:
            view.params['paginate_by'] = int(self.get['value'])
        except ValueError:
            pass
        return view


class ThisMonthItem(GetItem):
    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].filter(**{
            '%s__year' % self.field_name: datetime.today().year,
            '%s__month' % self.field_name: datetime.today().month
        })
        return view


class ThisWeekItem(GetItem):
    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].filter(**{
            '%s__gte' % self.field_name: (datetime.today() - \
                    timedelta(days=7)).strftime('%Y-%m-%d'),
            '%s__lt' % self.field_name: (datetime.today() + \
                    timedelta(days=1)).strftime('%Y-%m-%d'),
        })
        return view


class TagItem(GetItem):
    def __init__(self, request, title, get, field_name, tag, default=False):
        self.tag = tag
        super(TagItem, self).__init__(
            request=request,
            title=title,
            get=get,
            field_name=field_name,
            default=default
        )

    def modify(self, view):
        view.params['queryset'] = view.params['queryset'].filter(tags=self.tag)
        return view



from copy import copy
from inspect import getargspec
from functools import partial

from django import template
from django.template import TemplateDoesNotExist
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.template.loader import render_to_string
from django.template.context import Context
from django.template import Node, generic_tag_compiler, Variable
from django.template.base import Template, TagHelperNode
from django.utils.functional import curry
from django.utils.itercompat import is_iterable


register = template.Library()


# Base the inclusion_tag decorator on Django's default register tag. We want 
# to be able to dynamically compute a list of templates suitable for rendering.
def inclusion_tag(register, context_class=Context, takes_context=False, name=None):
    def dec(func):
        params, varargs, varkw, defaults = getargspec(func)

        class InclusionNode(TagHelperNode):

            def render(self, context):
                resolved_args, resolved_kwargs = self.get_resolved_arguments(context)
                
                # Only this line has been changed from the default
                # register_tag
                file_name, _dict = func(*resolved_args, **resolved_kwargs)

                if not getattr(self, 'nodelist', False):
                    from django.template.loader import get_template, select_template
                    if isinstance(file_name, Template):
                        t = file_name
                    elif not isinstance(file_name, basestring) and is_iterable(file_name):
                        t = select_template(file_name)
                    else:
                        t = get_template(file_name)
                    self.nodelist = t.nodelist
                new_context = context_class(_dict, **{
                    'autoescape': context.autoescape,
                    'current_app': context.current_app,
                    'use_l10n': context.use_l10n,
                    'use_tz': context.use_tz,
                })
                # Copy across the CSRF token, if present, because
                # inclusion tags are often used for forms, and we need
                # instructions for using CSRF protection to be as simple
                # as possible.
                csrf_token = context.get('csrf_token', None)
                if csrf_token is not None:
                    new_context['csrf_token'] = csrf_token
                return self.nodelist.render(new_context)

        function_name = (name or
            getattr(func, '_decorated_function', func).__name__)
        compile_func = partial(generic_tag_compiler,
            params=params, varargs=varargs, varkw=varkw,
            defaults=defaults, name=function_name,
            takes_context=takes_context, node_class=InclusionNode)
        compile_func.__doc__ = func.__doc__
        register.tag(function_name, compile_func)
        return func
    return dec


@register.inclusion_tag('jmbo/inclusion_tags/content_list_gizmo.html', \
        takes_context=True)
def content_list_gizmo(context, object_list):
    context.update({'object_list': object_list})
    return context


@inclusion_tag(register, takes_context=True)
def object_comments(context, obj):
    ctype = obj.content_type
    template_name = [
        "jmbo/inclusion_tags/%s/%s/object_comments.html" % (ctype.app_label, ctype.model),
        "jmbo/inclusion_tags/%s/object_comments.html" % ctype.app_label,
        "jmbo/inclusion_tags/object_comments.html"
    ]
    can_comment, code = obj.can_comment(context['request'])
    context.update({
        'object': obj,
        'can_render_comment_form': can_comment,
        'can_comment_code': code
        })
    return template_name, context


@register.inclusion_tag('jmbo/inclusion_tags/object_header.html', \
        takes_context=True)
def object_header(context, obj):
    context.update({'object': obj})
    return context


@register.inclusion_tag('jmbo/inclusion_tags/object_footer.html', \
        takes_context=True)
def object_footer(context, obj):
    context.update({'object': obj})
    return context


@register.tag
def pager(parser, token):
    """
    Output pagination links.
    """
    try:
        tag_name, page_obj = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            'pager tag requires 1 argument (page_obj), %s given' \
                    % (len(token.split_contents()) - 1)
            )
    return PagerNode(page_obj)


class PagerNode(template.Node):
    def __init__(self, page_obj):
        self.page_obj = template.Variable(page_obj)

    def render(self, context):
        page_obj = self.page_obj.resolve(context)
        context = {
            'request': context['request'],
            'page_obj': page_obj,
        }
        return render_to_string('jmbo/inclusion_tags/pager.html', context)


@register.tag
def render_object(parser, token):
    try:
        tag_name, obj, type = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            'render_object tag requires 2 arguments (obj, type), %s given' % \
                    (len(token.split_contents()) - 1)
            )
    return RenderObjectNode(obj, type)


class RenderObjectNode(template.Node):
    def __init__(self, obj, type):
        self.obj = template.Variable(obj)
        self.type = template.Variable(type)

    def render(self, context):
        obj = self.obj.resolve(context)
        type = self.type.resolve(context)

        # update context
        context = copy(context)
        context['object'] = obj

        # Generate template name from obj app label, model and type.
        obj_type = ContentType.objects.get_for_model(obj)
        template_name = "%s/inclusion_tags/%s_%s.html" % (obj_type.app_label, \
                obj_type.model, type)
        # Create response from template. if template is not found for obj type
        # use default content template. If default content template is not
        # found for type return empty response.
        try:
            response = render_to_string(template_name, context)
        except TemplateDoesNotExist:
            fallback_template_name = "jmbo/inclusion_tags/modelbase_%s.html" \
                    % type
            try:
                response = render_to_string(fallback_template_name, context)
            except TemplateDoesNotExist:
                if settings.TEMPLATE_DEBUG:
                    raise TemplateDoesNotExist({'resolved': template_name, \
                            'fallback': fallback_template_name})
                else:
                    response = ''

        return response


@register.tag
def view_modifier(parser, token):
    """
    Output view modifier.
    """
    try:
        tag_name, view_modifier = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            'view_modifier tag requires 1 argument (view_modifier), %s given' \
                    % (len(token.split_contents()) - 1)
            )
    return ViewModifierNode(view_modifier)


class ViewModifierNode(template.Node):
    def __init__(self, view_modifier):
        self.view_modifier = template.Variable(view_modifier)

    def render(self, context):
        view_modifier = self.view_modifier.resolve(context)
        context = {
            'request': context['request'],
            'view_modifier': view_modifier,
        }
        return render_to_string(
            'jmbo/inclusion_tags/view_modifier.html',
            context
        )



def as_div(form):
    """This formatter arranges label, widget, help text and error messages by
    using divs.  Apply to custom form classes, or use to monkey patch form
    classes not under our direct control."""
    # Yes, evil but the easiest way to set this property for all forms.
    form.required_css_class = 'required'

    return form._html_output(
        normal_row=u'<div class="field"><div %(html_class_attr)s>%(label)s %(errors)s <div class="helptext">%(help_text)s</div> %(field)s</div></div>',
        error_row=u'%s',
        row_ender='</div>',
        help_text_html=u'%s',
        errors_on_separate_row=False
    )



import copy
import traceback

from django.db.models import Q
from django.db.models.fields import related
from django.contrib import messages
from django.shortcuts import render_to_response
from django.template import loader
from django.template import RequestContext
from django.utils.translation import ugettext
from django.views.generic import list_detail


class DefaultURL(object):
    def __call__(self, obj=None):
        if obj:
            try:
                return obj.get_absolute_url()
            except AttributeError:
                return ''
        else:
            return self


class GenericBase(object):
    def __init__(self, *args, **kwargs):
        self.params = kwargs
        self.params['view_modifier'] = None
        self.params['url_callable'] = None

    def __call__(self, request, *args, **kwargs):
        self.request = request
        view = copy.copy(self)

        # setup view params
        view.params = view._resolve_view_params(
            request,
            view.defaults,
            *args,
            **kwargs
        )

        # push the view through view modifier
        if 'view_modifier' in view.params['extra_context']:
            view_modifier = view.params['extra_context']['view_modifier']
            if view_modifier:
                if callable(view_modifier):
                    view_modifier = view_modifier(request=request, \
                            *args, **kwargs)
                    view.params['extra_context']['view_modifier'] = \
                            view_modifier
                view = view_modifier.modify(view)

        # Apply possible limit after view modifiers. Django API requires
        # queryset slicing to occur last.
        limit = view.params.pop('limit', 0)
        if limit:
            view.params['queryset'] = view.params['queryset'][:limit]

        return view

    def get_url_callable(self, *args, **kwargs):
        return DefaultURL()

    def _resolve_view_params(self, request, defaults, *args, **kwargs):
        """
        Resolves view params with least ammount of resistance.
        Firstly check for params on urls passed args, then on class
        init args or members, and lastly on class get methods .
        """
        params = copy.copy(defaults)
        params.update(self.params)
        params.update(kwargs)
        resolved_params = {}

        extra_context = {}
        for key in params:
            # grab from provided params.
            value = params[key]

            # otherwise grab from existing params
            if value == None:
                value = self.params[key] if key in self.params else None

            # otherwise grab from class method
            if value == None:
                value = getattr(self, 'get_%s' % key)(request, *args, \
                        **kwargs) if getattr(self, 'get_%s' % key, None) \
                        else None

            if key in defaults:
                resolved_params[key] = value
            else:
                extra_context[key] = value

        if extra_context:
            try:
                resolved_params['extra_context'].update(extra_context)
            except AttributeError:
                resolved_params['extra_context'] = extra_context

        return resolved_params


class GenericObjectList(GenericBase):
    defaults = {
        'queryset': None,
        'paginate_by': None,
        'page': None,
        'allow_empty': True,
        'template_name': None,
        'template_loader': loader,
        'extra_context': None,
        'context_processors': None,
        'template_object_name': 'object',
        'mimetype': None,
        'limit': 0,
    }

    def __call__(self, request, *args, **kwargs):
        # generate our view via genericbase
        view = super(GenericObjectList, self).__call__(
            request,
            *args,
            **kwargs
        )

        # setup object_list params
        queryset = view.params['queryset']
        del view.params['queryset']

        # return object list generic view
        return list_detail.object_list(
            request,
            queryset=queryset,
            **view.params
        )

generic_object_list = GenericObjectList()


class GenericObjectFilterList(GenericObjectList):

    def __call__(self, request, *args, **kwargs):
        # generate our view via genericbase
        view = super(GenericObjectList, self).__call__(
            request,
            *args,
            **kwargs
        )

        # setup object_list params
        queryset = view.params['queryset']
        del view.params['queryset']

        # Filter
        for field in queryset.model._meta.fields:
            if field.name in view.params['extra_context'].keys():
                queryset = queryset.filter(Q(**{"%s__exact" % field.name: \
                        view.params['extra_context'][field.name]}))

        # return object list generic view
        return list_detail.object_list(
            request,
            queryset=queryset,
            **view.params
        )

generic_object_filter_list = GenericObjectFilterList()


class GenericObjectDetail(GenericBase):
    defaults = {
        'queryset': None,
        'object_id': None,
        'slug': None,
        'slug_field': 'slug',
        'template_name_field': None,
        'template_name': None,
        'template_loader': loader,
        'extra_context': None,
        'context_processors': None,
        'template_object_name': 'object',
        'mimetype': None,
    }

    def __call__(self, request, *args, **kwargs):
        # generate our view via genericbase
        view = super(GenericObjectDetail, self).__call__(
            request,
            *args,
            **kwargs
        )

        # setup object_list params
        queryset = view.params['queryset']
        del view.params['queryset']

        # return object list generic view
        return list_detail.object_detail(
            request,
            queryset=queryset,
            **view.params
        )

generic_object_detail = GenericObjectDetail()


class GenericForm(GenericBase):
    defaults = {
        'form_class': None,
        'form_args': {},
        'initial': None,
        'extra_context': None,
        'template_name': None,
        'success_message': None,
    }

    def handle_valid(self, form=None, *args, **kwargs):
        """
        Called after the form has validated.
        """
        # Take a chance and try save a subclass of a ModelForm.
        if hasattr(form, 'save'):
            form.save()
        # Also try and call handle_valid method of the form itself.
        if hasattr(form, 'handle_valid'):
            form.handle_valid(*args, **kwargs)

    def redirect(self, request, *args, **kwargs):
        """
        Redirect after successful form submission.
        Default behaviour is to not redirect and hence
        return the original view.
        """
        return None

    def __call__(self, request, *args, **kwargs):
        # generate our view via genericbase
        view = super(GenericForm, self).__call__(request, *args, **kwargs)

        self.form_class = view.params['form_class']
        self.form_args = view.params['form_args']
        self.initial = view.params['initial']
        self.template_name = view.params['template_name']
        self.success_message = view.params['success_message']

        if request.method == 'POST':
            form = self.form_class(
                data=request.POST,
                files=request.FILES,
                **self.form_args
            )
            if form.is_valid():
                self.handle_valid(form=form, request=request, *args, **kwargs)
                if self.success_message:
                    msg = ugettext(self.success_message)
                    messages.success(request, msg, fail_silently=True)
                redirect = self.redirect(request, *args, **kwargs)
                if redirect:
                    return redirect
        else:
            form = self.form_class(initial=self.initial, **self.form_args)

        context = RequestContext(request, {})
        context.update({
            'form': form,
        })
        context.update(view.params['extra_context'])
        return render_to_response(self.template_name, context)

generic_form_view = GenericForm()



from django.conf import settings
from django.db import models
from django.contrib.gis.db.models import GeoManager
from django.contrib.gis.db.models.query import GeoQuerySet


DB_ENGINE = settings.DATABASES['default']['ENGINE']


class LocationAwareQuerySet(GeoQuerySet):
    
    # annotates each object with a distance attribute
    def distance(self, point):
        # Join location attributes
        qs = self.select_related("location").exclude(location__coordinates__isnull=True)
        
        # Calculate spherical distance and store in distance attribute
        if DB_ENGINE.rfind('postgis') >= 0:
            sql = 'ST_Distance("atlas_location"."coordinates", ST_GeomFromText(\'%s\', %d))' \
                    % (str(point), point.srid)
        elif DB_ENGINE.rfind('mysql') >= 0:
            sql = 'distance_sphere(`atlas_location`.`coordinates`, geomfromtext(\'%s\', %d))' \
                    % (str(point), point.srid)
        elif DB_ENGINE.rfind('spatialite') >= 0:
            sql = 'ST_Distance("atlas_location"."coordinates", ST_GeomFromText(\'%s\', %d))' \
                    % (str(point), point.srid)
        else:
            raise ValueError("Distance calculations are not supported for ModelBase using this database.")
        
        return qs.extra(select={'distance': sql})


class LocationAwareManager(GeoManager):
    def get_query_set(self):
        return LocationAwareQuerySet(self.model)
        
    
class PermittedManager(LocationAwareManager):
    def get_query_set(self):
        # Get base queryset and exclude based on state.
        queryset = super(PermittedManager, self).get_query_set().exclude(
            state='unpublished'
        )

        # Exclude objects in staging state if not in
        # staging mode (settings.STAGING = False).
        if not getattr(settings, 'STAGING', False):
            queryset = queryset.exclude(state='staging')

        # Filter objects for current site.
        queryset = queryset.filter(sites__id__exact=settings.SITE_ID)

        return queryset


class DefaultManager(LocationAwareManager):

    def get_by_natural_key(self, slug):
        return self.get(slug=slug)
